package main

import (
	"encoding/base64"
	"flag"
	"fmt"
	"golang.org/x/crypto/ssh"
	"log"
	"os"
)

const (
	Principals  = "p"
	KeyId       = "k"
	Serial      = "s"
	ValidBefore = "v"
)

var (
	logFolderPath      string
	logFileLocation    string
	returnOutput       string
	defaultRoleName    string
	encodedCertificate string
	logFileName        string
)

func main() {
	/*
		This script accepts a Base64 encoded string from a runtime flag containing an OpenSSH Wire-Format (RFC 4251) SSH Certificate.
		This script decodes the Base64 encoded string and can be configured to return its principals output based on the
	*/

	// Parse runtime options
	flag.StringVar(&logFolderPath, "logsdir", "/var/log/banyan", "The folder where log output will be written")
	flag.StringVar(&logFileName, "logfilename", "jitsu-certificate_decoder.log", "The file where log output will be written")
	flag.StringVar(&returnOutput, "output", "p", "The decoded output type to return (Principals: -output=p, KeyId: -output=k, Serial: -output=s, ValidBefore: -output=v")
	flag.StringVar(&defaultRoleName, "default_role", "ANY", "The default role that all users are assigned to")
	flag.StringVar(&encodedCertificate, "cert", "BASE64", "The Base64 encoded string containing an OpenSSH Wire-Format (RFC 4251) SSH Certificate")
	flag.Parse()

	// Create logFolderPath if it does not already exist
	if _, err := os.Stat(logFolderPath); os.IsNotExist(err) {
		os.Mkdir(logFolderPath, os.ModeDir)
	}

	logFileLocation = fmt.Sprintf("%s/%s", logFolderPath, logFileName)

	// Open Log File
	logFile, err := os.OpenFile(logFileLocation, os.O_RDWR|os.O_CREATE|os.O_APPEND, 0666)
	if err != nil {
		log.Fatalf("Error opening logFile %s with error: %s", logFileLocation, err.Error())
	}

	defer logFile.Close()

	log.SetOutput(logFile)
	log.Printf("Certificate Decoder Started")

	// Decode Base64 String to WireFormat Byte Array
	decodedCertificate, err := base64.StdEncoding.DecodeString(encodedCertificate)
	if err != nil {
		log.Fatalf("Error decoding Base64 encoded SSH Certificate: %s", err.Error())
	}

	// Parsing Public Key to ssh.PublicKey struct
	parsedCertificate, err := ssh.ParsePublicKey(decodedCertificate)
	if err != nil {
		log.Fatalf("Error parsing Wire SSH Certificate into SSH Certificate: %s", err.Error())
	}

	// Converting ssh.PublicKey to ssh.Certificate
	convertedCertificate := parsedCertificate.(*ssh.Certificate)

	// Returning proper output to STDOUT
	switch returnOutput {
	case KeyId:
		log.Printf("Returning Key ID: %#v", convertedCertificate.KeyId)
		fmt.Printf("%s", convertedCertificate.KeyId)
	case Serial:
		log.Printf("Returning Serial ID: %#v", convertedCertificate.Serial)
		fmt.Printf("%d", convertedCertificate.Serial)
	case ValidBefore:
		log.Printf("Returning Valid Before: %#v", convertedCertificate.ValidBefore)
		fmt.Printf("%d", convertedCertificate.ValidBefore)
	case Principals:
		fallthrough
	default:
		log.Printf("Returning Valid Principals: %#v", convertedCertificate.ValidPrincipals)
		for _, x := range convertedCertificate.ValidPrincipals {
			if x != defaultRoleName { // Don't print the defaultRoleName principal
				fmt.Printf("%s\n", x)
			}
		}
	}
}
