#!/bin/bash

## This Script will set up or reset Jitsu
## Run with sudo ./jitsu-service.sh [ start | stop ]

## Return if an error occurs
set -e

## Defaults
log_dir="/var/log/banyan"
log_file="jitsu-service.log"
sshd_config_path="/etc/ssh/sshd_config"
ssh_ca_path="/etc/ssh/banyan_ca.pub"
jitsu_dir="/opt/banyan"

## Load Jitsu configuration
source "$jitsu_dir/jitsu.conf"

## Logger
function log () {
  echo "$(date) $1" >> "$log_dir/$log_file"
}

## Root Check
function su_check () {
  if [ "$EUID" -ne 0 ]; then 
    echo "Jitsu must be run as root"
    exit 1
  fi
}

## Create Log Directory if it does not already exist
function create_log_dir () {
  mkdir -p "$log_dir"
}

function stop_libnss () {
  log "Stopping LIBNSS"
  sudo /bin/cp -f "$jitsu_dir/nsswitch.conf.noato" /etc/nsswitch.conf
}

function start_libnss () {
  log "Starting LIBNSS"
  sudo /bin/cp -f "$jitsu_dir/nsswitch.conf.ato" /etc/nsswitch.conf
}

## Make Sure sshd_config Includes Jitsu SSHD Config 
function clear_sshd_config () {
  log "Clearing exising Jitsu Configs from $sshd_config_path"
  sed -i.bak \
  -e "s|.*UsePAM.*||g" \
  -e "s|.*TrustedUserCAKeys\s*${ssh_ca_path}.*||g" \
  -e "s|.*AuthorizedPrincipalsCommandUser\s*root.*||g" \
  -e "s|.*AuthorizedPrincipalsCommand\s*${jitsu_dir}.*||g" \
  -e "/^[[:space:]]*$/d" $sshd_config_path > /tmp/sshd_config_backup
}

## Update the SSHD Config for Jitsu
function update_sshd_config () {
  log "Updating SSHD Config for Jitsu"
  echo "UsePAM yes" >> $sshd_config_path
  echo "TrustedUserCAKeys $ssh_ca_path" >> $sshd_config_path
  echo "AuthorizedPrincipalsCommandUser root" >> $sshd_config_path
  echo "AuthorizedPrincipalsCommand $jitsu_dir/jitsu.sh %u %i %k" >> $sshd_config_path
  sudo systemctl restart sshd ## Restart SSHD
}

if [ "$1" = "start" ]; then
  echo "Jitsu Service Starting"
  log "Jitsu Service Starting"
  su_check
  create_log_dir
  if [ "$MODE" = "DEFAULT" ] || [ "$MODE" = "NO_GROUPS" ]; then
    start_libnss
  fi 
  clear_sshd_config
  update_sshd_config
  echo "Jitsu Service Started"
  log "Jitsu Service Started"
elif [ "$1" = "stop" ]; then
  echo "Jitsu Service Stopping"
  log "Jitsu Service Stopping"
  su_check
  create_log_dir
  if [ "$MODE" = "DEFAULT" ] || [ "$MODE" = "NO_GROUPS" ]; then
    stop_libnss
  fi 
  clear_sshd_config
  echo "Jitsu Service Stopped"
  log "Jitsu Service Stopped"
else
  su_check
  create_log_dir
  log "Exiting immediately"
fi

exit 0
