#!/bin/bash

## This Script is the Jitsu runtime. 
## It is called by OpenSSH to retrieve required roles for SSH authentication requests. 
## Its configuration is stored in $jitsu_dir/jitsu.conf

## Return if an error occurs
#set -euo pipefail

## Defaults
log_dir="/var/log/banyan"
log_file="jitsu.log"
jitsu_dir="/opt/banyan"
default_roles_file="default.roles"
root_roles_file="root.roles"

## Load Jitsu configuration
source "$jitsu_dir/jitsu.conf"

trap "sudo /bin/cp -f $jitsu_dir/nsswitch.conf.ato /etc/nsswitch.conf" EXIT

## Logger
function log () {
  echo "$(date) $1" >> "$log_dir/$log_file"
}

## Root Check
function su_check () {
  if [ "$EUID" -ne 0 ]; then 
    log "Jitsu must be run as root"
    exit 1
  fi
}

## Retreive SSH CA Certficiate
function get_roles () {
  if [ -n "$POLICY" ]; then
    ## Retrieve roles from Banyan based on Policy
    log "Retreiving Roles from Banyan"
  else
    ## Check if the user is root
    if [ "$USERNAME" = "root" ]; then
      ## Retrieve roles from $jitsu_dir/$root_roles_file or 
      log "Reading root roles from $jitsu_dir/$root_roles_file"
      while read -r line
      do
        echo "$line"
      done < "$jitsu_dir/$root_roles_file"
    else
      if [ "$USERNAME" = "$KEY_ID" ]; then
        log "Retreiving roles for $USERNAME"
        if [ -z "$USER_EXISTS" ]; then ## If new user, create user account and home dir
          log "User does not currently exist, creating it"
          if [ "$MODE" = "DEFAULT" ]; then
            ## Decode the Principals from the Certificate
            OLD_IFS=$IFS
            IFS=$'\n'
            PRINCIPALS=()
            for PRINCIPAL in $($jitsu_dir/certificate_decoder -cert "$CERTIFICATE")
            do
              GROUP_EXISTS=$(grep "^$PRINCIPAL:" /etc/group)
              if [ -z "$GROUP_EXISTS" ]; then 
                log "Creating group: ${PRINCPAL}"
                sudo groupadd "$PRINCIPAL"
              fi
              PRINCIPALS+=( "$PRINCIPAL" )
            done
            IFS=$OLD_IFS
            NUM_PRINCIPALS=${#PRINCIPALS[@]}
            if [[ $NUM_PRINCIPALS -gt 1 ]]; then
              SECONDARY_PRINCIPALS=("${PRINCIPALS[@]:1}")
              SECONDARY_PRINCIPALS_LIST=$( IFS=$','; echo "${SECONDARY_PRINCIPALS[*]}" )
            fi
            IFS=$OLD_IFS
            log "Finished retreiving principals"
          fi
          if [ "$MODE" = "DEFAULT" ] || [ "$MODE" = "NO_GROUPS" ]; then
            log "Switching off LIBNSS"
            sudo /bin/cp -f "$jitsu_dir/nsswitch.conf.noato" /etc/nsswitch.conf
            if [ "$MODE" = "DEFAULT" ]; then
              if [[ $NUM_PRINCIPALS -gt 1 ]]; then
                log "Creating User With Groups ${PRINCIPALS[0]} and $SECONDARY_PRINCIPALS_LIST for $USERNAME"
                sudo useradd -m -g "${PRINCIPALS[0]}" -G "$SECONDARY_PRINCIPALS_LIST" -s "/bin/bash" "$USERNAME"
              elif [[ $NUM_PRINCIPALS -eq 1 ]]; then
                log "Creating User With Groups ${PRINCIPALS[0]} and no secondary groups for $USERNAME"
                sudo useradd -m -g "${PRINCIPALS[0]}" -s "/bin/bash" "$USERNAME"
              else 
                log "Creating User without Groups for $USERNAME"
                sudo useradd -m -s "/bin/bash" "$USERNAME"
              fi
            else
              log "Creating User Without Groups for $USERNAME"
              sudo useradd -m -s "/bin/bash" "$USERNAME"
            fi
            log "Switching on LIBNSS"
            sudo /bin/cp -f "$jitsu_dir/nsswitch.conf.ato" /etc/nsswitch.conf    
          fi
        fi
        log "Reading default roles from $jitsu_dir/$default_roles_file"
        while read -r line
        do
          echo "$line"
        done < "$jitsu_dir/$default_roles_file"
      else
        log "Incorrect USERNAME: $USERNAME for KEY_ID: $KEY_ID"
        echo ""
      fi
    fi
  fi
}

if [[ $# -ge 3 ]]; then
  log "Running in mode $MODE"
  USERNAME="${1}"
  KEY_ID=$(echo "${2}" | sed 's/@.*//')
  USER_EXISTS=$(grep "^$USERNAME:" /etc/passwd || true)
  CERTIFICATE="${3}"
  su_check
  get_roles
else
  log "Incorrect number of arguments given: $#, expected 3"
  exit 1
fi

exit 0