#!/bin/bash

## This Script will install Jitsu
## Run with sudo ./jitsu-installer.sh
##
## Optional Environment Variables:
## - NO_GROUPS: The NO_GROUPS variable can be set to disable JIT Groups Creation
## - NO_USERS: The NO_USERS variable can be sset to disable JIT User Creation (includes NO_GROUPS)
## - REFRESH_TOKEN: The REFRESH_TOKEN variable can be set to retreive the SSH CA for the server from Banyan
## - TRUST_PROVIDER: The TRUST_PROVIDER variable can be set to retrieve the SSH CA for the server from Banyan
## - POLICY: The POLICY variable can be set to retreive the roles for accessing the server from Banyan

## Return if an error occurs
set -e

## Defaults
log_dir="/var/log/banyan"
log_file="jitsu-installer.log"
sshd_config_path="/etc/ssh/sshd_config"
ssh_ca_path="/etc/ssh/banyan_ca.pub"
jitsu_dir="/opt/banyan"
default_roles_file="default.roles"
root_roles_file="root.roles"

## Download Links
JITSU_RUNTIME="" ## TODO
JITSU_SERVICE="" ## TODO
JITSU_SERVICE_RUNTIME="" ## TODO
CERTIFICATE_DECODER="" ## TODO
LIBNSS_RUNTIME="" ## TODO

## Logger
function log () {
  echo "$(date) $1" >> "$log_dir/$log_file"
}

## Root Check
function su_check () {
  if [ "$EUID" -ne 0 ]; then 
    echo "Jitsu must be run as root"
    exit 1
  fi
}

## Create Log Directory if it does not already exist
function create_log_dir () {
  echo "Creating log directory if it does not already exist"
  mkdir -p "$log_dir"
}


## Create Jitsu Directory if it does not already exist
function create_jitsu_dir () {
  echo "Creating jitsu directory if it does not already exist"
  mkdir -p "$jitsu_dir"
}

## Will Install the prerequisites for running the SSH CA
function install_prerequisites () {
  log "Installing the JITSU Runtime from $JITSU_RUNTIME"
  # $DOWNLOAD "./jitsu.sh" "$JITSU_RUNTIME" ## Download Jitsu runtime
  sudo /bin/cp -f "./jitsu.sh" "$jitsu_dir/jitsu.sh" ## Move jitsu runtime to $jitsu_dir/jitsu.sh
  sudo chmod u+x "$jitsu_dir/jitsu.sh" ## Make Jitsu runtime executable

  log "Installing the JITSU Service from $JITSU_SERVICE"
  # $DOWNLOAD "./jitsu.service" "$JITSU_SERVICE" ## Download Jitsu service
  sudo /bin/cp -f "./jitsu.service" /etc/systemd/system/jitsu.service ## Move jitsu service to /etc/systemd/system/jitsu.service

  log "Installing the JITSU Service Runtime from $JITSU_SERVICE_RUNTIME"
  # $DOWNLOAD "./jitsu-service.sh" "$JITSU_SERVICE_RUNTIME" ## Download Jitsu service runtime
  sudo /bin/cp -f "./jitsu-service.sh" "$jitsu_dir/jitsu-service.sh" ## Move jitsu service runtime to $jitsu_dir/jitsu-service.sh
  sudo chmod u+x "$jitsu_dir/jitsu-service.sh" ## Make Jitsu service runtime executable

  if [ -z "$NO_USERS" ]; then ## Download LIBNSS if NO_USERS is unset
    if command -v apt-get >/dev/null; then
      log "Installing libnss from $LIBNSS_RUNTIME.deb"
      # $DOWNLOAD "./libnss.deb" "$LIBNSS_RUNTIME.deb"
      sudo dpkg -i ./bin/libnss.deb
    else
      log "Installing libnss from $LIBNSS_RUNTIME.rpm"
      # $DOWNLOAD "./libnss.rpm" "$LIBNSS_RUNTIME.rpm"
      sudo rpm –i ./bin/libnss.rpm
    fi

    if [ -z "$NO_GROUPS" ]; then  ## Download CERTIFICATE_DECODER if NO_GROUPS is also unset
      log "Installing certificate_decoder from $CERTIFICATE_DECODER"
      # $DOWNLOAD "./certificate_decoder" "$CERTIFICATE_DECODER"
      sudo /bin/cp -f "./bin/certificate_decoder_linux_x86" "$jitsu_dir/certificate_decoder"
      sudo chmod u+x "$jitsu_dir/certificate_decoder"
    fi
  fi
}

## Curl or Wget Checker
function curl_or_wget () {
  if hash curl 2>/dev/null; then
    DOWNLOAD="curl -o"
  elif hash wget 2>/dev/null; then
    DOWNLOAD="wget -O"
  else 
    log "Neither curl nor wget found. Unable to retreive prerequisites."
    exit 1
  fi
}

## Retreive SSH CA Certficiate
function get_ssh_ca () {
  ## TODO: Place SSH CA in $ssh_ca_path
  echo "Please Place the Banyan SSH CA from the Command Center at $ssh_ca_path"
  log "Please Place the Banyan SSH CA from the Command Center at $ssh_ca_path"
}

## Set up NSSwitch in LIBNSS is installed
function libnss_setup() {
  log "Starting LIBNSS Setup"

  log "Creating provisioner user"
  sudo useradd -m provisioner ## Create a provisioner user
  su -c "mkdir -p ~/.ssh" provisioner

cat >/tmp/rc <<EOF
echo "--------------------------------------------"
echo ""
echo "Your user has been successfully provisioned!"
echo ""
echo "Please log in again to access this server."
echo ""
echo "--------------------------------------------"
pkill -KILL -u provisioner
EOF

  su -c "/bin/cp -f /tmp/rc ~/.ssh/rc" provisioner
  grep "^provisioner" /etc/passwd > /etc/libnss-ato.conf

  if [ -f "$jitsu_dir/nsswitch.conf.noato" ]; then
    log "ATO Conf already exists, overwriting"
    sudo /bin/cp -f "$jitsu_dir/nsswitch.conf.noato" /etc/nsswitch.conf
  fi

  sudo /bin/cp -f /etc/nsswitch.conf "$jitsu_dir/nsswitch.conf.noato"
  sed -e '/^passwd/s/$/ ato/' -e '/^shadow/s/$/ ato/' "$jitsu_dir/nsswitch.conf.noato" > "$jitsu_dir/nsswitch.conf.ato"
  sudo /bin/cp -f "$jitsu_dir/nsswitch.conf.ato" /etc/nsswitch.conf
}

## Make Sure sshd_config Includes Jitsu SSHD Config 
function clear_sshd_config () {
  log "Clearing exising Jitsu Configs from $sshd_config_path"
  sed -i.bak \
  -e "s|.*UsePAM.*||g" \
  -e "s|.*TrustedUserCAKeys\s*${ssh_ca_path}.*||g" \
  -e "s|.*AuthorizedPrincipalsCommandUser\s*root.*||g" \
  -e "s|.*AuthorizedPrincipalsCommand\s*${jitsu_dir}.*||g" \
  -e "/^[[:space:]]*$/d" $sshd_config_path > /tmp/sshd_config_backup
}

## Update the SSHD Config for Jitsu
function update_sshd_config () {
  log "Updating SSHD Config for Jitsu"
  echo "UsePAM yes" >> $sshd_config_path
  echo "TrustedUserCAKeys $ssh_ca_path" >> $sshd_config_path
  echo "AuthorizedPrincipalsCommandUser root" >> $sshd_config_path
  echo "AuthorizedPrincipalsCommand $jitsu_dir/jitsu.sh %u %i %k" >> $sshd_config_path
  sudo systemctl restart sshd ## Restart SSHD
}

function set_jitsu_config () {
  log "Updating Jitsu Config with ENV Varibles: $NO_GROUPS, $NO_USERS, $REFRESH_TOKEN, $TRUST_PROVIDER, $POLICY"
  if [ -n "$NO_GROUPS" ]; then
    echo "MODE=NO_GROUPS" > "$jitsu_dir/jitsu.conf"
  elif [ -n "$NO_USERS" ]; then
    echo "MODE=NO_USERS" > "$jitsu_dir/jitsu.conf"
  else
    echo "MODE=DEFAULT" > "$jitsu_dir/jitsu.conf"
  fi

  if [ -n "$REFRESH_TOKEN" ]; then
    log "Setting REFRESH_TOKEN=${REFRESH_TOKEN}"
    echo "REFRESH_TOKEN=${REFRESH_TOKEN}" >> "$jitsu_dir/jitsu.conf"
  fi

  if [ -n "$TRUST_PROVIDER" ]; then
    log "Setting TRUST_PROVIDER=${TRUST_PROVIDER}"
    echo "TRUST_PROVIDER=${TRUST_PROVIDER}" >> "$jitsu_dir/jitsu.conf"
  fi

  if [ -n "$POLICY" ]; then
    log "Setting POLICY=${POLICY}"
    echo "POLICY=${POLICY}" >> "$jitsu_dir/jitsu.conf"
  fi
}

su_check
create_log_dir
create_jitsu_dir
get_ssh_ca
clear_sshd_config
update_sshd_config
set_jitsu_config
curl_or_wget
install_prerequisites
if [ -z "$NO_USERS" ]; then
  libnss_setup
fi
log "Starting Jitsu service"
sudo systemctl start jitsu
log "Enabling Jitsu service"
sudo systemctl enable jitsu
log "JITSU installed sucessfully"
echo "JITSU has been installed in the ${jitsu_dir} directory"
if [ -z "$POLICY" ]; then
  echo "You must create the $jitsu_dir/$default_roles_file file for non-root roles, and the $jitsu_dir/$root_roles_file file for root roles"
fi

exit 0